''APRS Transmitter

''In theory should work with most handhelt 2m radios tuned to APRS frequency.
''Uses code written by Phil Pilgrim and G3CWI-Richard

CON''----------------------------------------------------------------------------------------

  _clkmode = xtal1 + pll2x                                                         
  _xinfreq = 5_000_000

'Transmiter
  TX_SIG       = 22
  TX_ENA       = 23
                                
'TP Sensor
  SCL           = 19
  SDA           = 18

'GPS
  GPS_SIG       = 3

 
VAR''----------------------------------------------------------------------------------------

' byte = 256                    'Capable of "+" values only
' word = 65,536                 'Capable of "+" values only
' long = 4,294,967,296          'Capable of both "+" and "-" values

'GPS
  Long  Knots                                                
  Long  HghLat
  Long  LowLat
  Long  HghLng
  Long  LowLng
  Long  Alt
  Word  Head
  Word  Year
  Byte  Month
  Byte  Day
  Byte  Hour
  Byte  Minute
  Byte  Second

' Sensors
  Long  MS_Pa
  Long  MS_degC 

' Basic Data
  Long  Sigma

' Transmitter
  Long  CallSign
  Word  PacketSize
  Byte  SSID
 
' Watchdog
  Long  stack[20]
  Byte  Watchdog_Cog
  Byte  Watchdog_Success


OBJ''----------------------------------------------------------------------------------------

  Packet        : "APRS_Packet_Generator_V35"  
  GPS           : "GPS_Light_V1"         
  Debug         : "FullDuplexSerial"
  MS5607        : "MS5607"
     
  
PUB Start''----------------------------------------------------------------------------------

  Watchdog_Success := (watchdog_cog := cognew(Watchdog(@Sigma), @stack)+1) 

  CallSign := string("XX5XXX")                          'Amateur Radio Call Sign
  SSID     := 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               '************** 

'Initialize Sensor  
  MS5607.Start(SDA, SCL)

'Initialize GPS
  GPS.Start(GPS_SIG)  
  
  Main


PUB Main''-----------------------------------------------------------------------------------

  repeat

    'Transmitts position, temperatre, and pressure once every minute.

    Get_GPS_Data                                      

    'Get Sensor Data
    MS5607.Read
    MS_degC := MS5607.Temperature/10                        
    MS_Pa   := MS5607.Pressure

    'Transmit
    Transmit_Data
    Sigma := Sigma + 1

    Display_Data                'For debug only. Comment out otherwise.

    wait(60)


PUB Transmit_Data''--------------------------------------------------------------------------

'Creates APRS packet, enables transmitter, transmits, and disables transmitter.

  Packet.Generate(CallSign, SSID, HghLat, LowLat, HghLng, LowLng, Head, Knots, Alt)
  Packet.Add_Data(MS_degC, MS_Pa, Sigma)   
  
  TXR_ENA                                                        
  PacketSize := Packet.Transmit(TX_SIG)                                                     
  TXR_DSA
  

PUB Get_GPS_Data''--------------------------------------------------------------------------- 
  
  HghLat   := GPS.HighLat
  LowLat   := GPS.LowLat                                'To the ten thousandth's place (.0000)         
  HghLng   := GPS.HighLong
  LowLng   := GPS.LowLong                               'To the ten thousandth's place (.0000)
  Head     := GPS.Head                                  'Get Course over ground in deg 
  Knots    := GPS.Speed                                 'Get Speed in knots
  Alt      := GPS.Altitude                              'Get Altitude in meters 
  Year     := GPS.Year                                  'Get Year
  Month    := GPS.Month                                 'Get Month
  Day      := GPS.Day                                   'Get Day     
  Hour     := GPS.Hour                                  'Get Hour      
  Minute   := GPS.Minute                                'Get Minute
  Second   := GPS.Second                                'Get Second


PUB TXR_ENA''--------------------------------------------------------------------------------

  dira[TX_ENA] := 1
  outa[TX_ENA] := 1

  
PUB TXR_DSA''--------------------------------------------------------------------------------

  outa[TX_ENA] := 0
  dira[TX_ENA] := 0

  
PUB Wait(time)''-----------------------------------------------------------------------------

  waitcnt(clkfreq*time + cnt)


PUB Display_Data''---------------------------------------------------------------------------

  Debug.start(31, 30, 0, 57600)
        
  waitcnt(clkfreq/10 + cnt)

  Display_Computed_Data

  waitcnt(clkfreq/5 + cnt)                                 
  debug.stop                                                 

      
PUB Display_Computed_Data''------------------------------------------------------------------

  debug.str(string("-Data--------------------", 13))    ' 25 characters long

  debug.str(string(" GPS Date     "))
  debug.dec(Month)
  debug.str(string("/"))
  debug.dec(Day)              
  debug.str(string("/")) 
  debug.dec(Year)    
  debug.str(string("    ", 13))
  
  debug.str(string(" GPS Time     "))
  debug.dec(Hour)
  debug.str(string(":"))
  debug.dec(Minute)              
  debug.str(string(":")) 
  debug.dec(Second)    
  debug.str(string("    ", 13))

  debug.str(string(" Latitude     "))                     
  debug.dec(HghLat)                                               
  debug.str(string(".")) 
  debug.dec(Lowlat)                                             
  debug.str(string("    ", 13))  

  debug.str(string(" Longitude    "))                    
  debug.dec(HghLng)                                               
  debug.str(string(".")) 
  debug.dec(LowLng)                                                  
  debug.str(string("    ", 13)) 

  debug.str(string(" Alt          "))            
  debug.dec(Alt)
  debug.str(string("    ", 13))    

  debug.str(string(" Temp         "))             
  debug.dec(MS_degC)                                               
  debug.str(string("    ", 13))

  debug.str(string(" Press        "))             
  debug.dec(MS_Pa)                                               
  debug.str(string("    ", 13))  

  debug.str(string(" Sigma        "))             
  debug.dec(Sigma)                                               
  debug.str(string("    ", 13))

  debug.str(string("    ", 13))            


PRI Watchdog(Sigma_Address)| Sigma_Slave, Timer, Set_Time''----------------------------------

'In the Watchdog method, the Sigma_Slave variable increments along with Sigma.
'As long as the Sigma_Slave variable increments before the Timer expires, 
'the Watchdog method will not reboot the system. 

'Set Time limit in seconds
  Set_Time := 300  

  Sigma_Slave := long[Sigma_Address]
  Timer := Set_Time
  
  repeat
' Wait for Sigma to increment or Timer to expire, whichever comes first.  
    repeat while Sigma_Slave == long[Sigma_Address] and Timer > 0  
      waitcnt(clkfreq*1 + cnt)     
      Timer := Timer - 1

' If we exited the repeat loop because the Timer expired, reboot system.
    If Timer == 0
      reboot

' Else we exited the repeat loop because Sigma incremented, update Sigma_Slave and reset Timer.
    Timer := Set_Time  
    Sigma_Slave := long[Sigma_Address]      
 

''-------------------------------------------------------------------------------------------     