''File: MS5607


CON''----------------------------------------------------------------------------------------  

  _clkmode = xtal1 + pll16x                                                         
  _xinfreq = 5_000_000

'These constants provide address and command byte values for the sensor.  
  DEVICE_ADDR      = $ec
  DEVICE_READ      = $01
  CMD_READ_ADC     = $00
  CMD_RESET        = $1e             
  CMD_CONV_ADC     = $40
  CMD_READ_PROM    = $a0
  RSLT_D1          = $00
  RSLT_D2          = $10      

'These constants provide different rates for the sensor.
  RES_256          = $00        ' Lowest
  RES_512          = $02        ' Low
  RES_1024         = $04        ' Medium
  RES_2048         = $06        ' High
  RES_4096         = $08        ' Highest

  
VAR''---------------------------------------------------------------------------------------- 

  long adc_delay, MS5607_Temp, MS5607_Press  
  word c1, c2, c3, c4, c5, c6, ptr
  byte sda, scl, resolution, dev5611
  

PUB Read

  Read_MS5607

PUB Pressure : mbar

  mbar := MS5607_Press 
  
PUB Temperature : degC

  degC := MS5607_Temp 
 
PUB Start(sd, sc) | i

  set_resolution(RES_1024)
  scl := sc & $1f
  sda := sd & $1f
  dev5611 := (sc | sd) & $80
  ptr~
  _i2c_init                                             ' Resynchronize the I2C interface after power up or a reset.
  _i2c_send_cmd(CMD_Reset)                              ' Send a cmd to the sensor
  waitcnt((clkfreq >> 3 #> 400) + cnt)
  repeat i from 0 to 5
    c1[i] := _read_prom(i + 1)

PUB set_resolution(resol) | i

  if (i := lookdown(resol: RES_256, RES_512, RES_1024, RES_2048, RES_4096))
    resolution := resol
    adc_delay := clkfreq / lookup(i: 1000, 333, 250, 166, 100) #> 400


PUB Read_MS5607 | d1, d2, dt, offh, offl, sensh, sensl, ph, pl

  d1 := _read_adc(RSLT_D1)
  d2 := _read_adc(RSLT_D2)
  
  dt := d2 - c5 << 8
  MS5607_Temp := 2000 + ((dt ** c6) << 9 + (dt * c6) >> 23)

  offh := c4 ** dt
  offl := (c4 * dt) >> 6 | offh << 26
  offh ~>= 6
  _dadd(@offh, c2 >> 15, c2 << 17)

  sensh := c3 ** dt
  sensl := (c3 * dt) >> 7 | sensh << 25
  sensh ~>= 7
  _dadd(@sensh, c1 >> 16, c1 << 16)

  _umult(@ph, d1, sensl)
  ph += d1 * sensh
  pl := pl >> 21 | ph << 11
  ph ~>= 21
  _dsub(@ph, offh, offl)
  MS5607_Press := ph << 17 | pl >> 15

PRI _read_prom(addr)

  {{ Read one word of data from the sensor's PROM at the address specified.
  }}

  return _i2c_receive_data(CMD_READ_PROM | addr << 1, 2)

PRI _read_adc(which) | d

  {{ Start a temperature or pressure conversion, then read the 24-bit value after
     a resolution-dependent delay.
  }}

  _i2c_send_cmd(CMD_CONV_ADC | which | resolution)
  waitcnt(adc_delay + cnt)
  return _i2c_receive_data(CMD_READ_ADC, 3)

''-------[ I2C Methods... ]----------------------------------------------------

PRI _i2c_init

  {{ Resynchronize the I2C interface after power up or a reset.
  }}

  repeat 8
    dira[scl]~
    dira[scl]~~ 
    dira[scl]~

PRI _i2c_receive_data(cmd, nbytes) : value | i

  {{ Send a read command and read `nbytes (1 - 4) bytes of data into the return long.
  }}

  _i2c_send_cmd(cmd)
  _i2c_start
  _i2c_send(DEVICE_ADDR | DEVICE_READ)
  repeat i from 1 to nbytes
    value := value << 8 | _i2c_receive(true & (i <> nbytes))
  _i2c_stop

PRI _i2c_send_cmd(cmd)

  {{ Send a command to the sensor.
  }}

  _i2c_start
  _i2c_send(DEVICE_ADDR)
  _i2c_send(cmd)
  _i2c_stop

PRI _i2c_send(value)

  {{ Send a byte to the sensor.
  }}

  value := ((!value) >< 8)

  repeat 8
    dira[sda] := value
    dira[scl] := false
    dira[scl] := true
    value >>= 1

  dira[sda] := false
  dira[scl] := false
  result := not(ina[sda])
  dira[scl] := true
  dira[sda] := true

PRI _i2c_receive(aknowledge)

  {{ Receive a byte from the sensor.
  }}

  dira[sda] := false

  repeat 8
    result <<= 1
    dira[scl] := false
    result |= ina[sda]
    dira[scl] := true

  dira[sda] := (aknowledge)
  dira[scl] := false
  dira[scl] := true
  dira[sda] := true

PRI _i2c_start

  {{ Send an I2C start condition.
  }}

  outa[sda] := false
  outa[scl] := false
  dira[sda] := true
  dira[scl] := true

PRI _i2c_stop

  {{ Send an I2C stop condition.
  }}

  dira[scl] := false
  dira[sda] := false

''-------[ Double Long Math... ]--------------------------------------------------

{{ These methods perform 64-bit, unsigned interger math operations.
}} 

PRI _umultdiv(x, num, denom) | producth, productl

  {{ Multiply `x and `num, then divide the resulting unsigned, 64-bit product by
     `denom to yield a 32-bit unsigned result. `x ** `num must be less than (unsigned) `denom.
  }}

  _umult(@producth, x, num)
  return _udiv(producth, productl, denom)   

PRI _umult(productaddr, mplr, mpld) | producth, productl

  {{ Multiply `mplr by `mpld and store the unsigned 64-bit product at `productaddr.
  }}

  producth := (mplr & $7fff_ffff) ** (mpld & $7fff_ffff)
  productl := (mplr & $7fff_ffff) * (mpld & $7fff_ffff)
  if (mplr < 0)
    _dadd(@producth, mpld >> 1, mpld << 31)
  if (mpld < 0)
    _dadd(@producth, mplr << 1 >> 2, mplr << 31)
  longmove(productaddr, @producth, 2)

PRI _udiv(dvndh, dvndl, dvsr) | carry, quotient

  {{ Divide the unsigned 64-bit number `dvndh:`dvndl by `dvsr, returning an unsigned 32-bit quotient.
     Saturate result to $ffff_ffff if it's too big to fit 32 bits.
  }}

  quotient~
  ifnot (_ult(dvndh, dvsr))
    return $ffff_ffff
  repeat 32
    carry := dvndh < 0
    dvndh := (dvndh << 1) + (dvndl >> 31)
    dvndl <<= 1
    quotient <<= 1
    if (not _ult(dvndh, dvsr) or carry)
      quotient++
      dvndh -= dvsr
  return quotient

PRI _dsub(difaddr, subh, subl)

  {{ Subtract the 64-bit value `subh:`subl from the 64-bit value stored at `difaddr.
  }}
  
  _dadd(difaddr, -subh - 1, -subl)

PRI _dadd(sumaddr, addh, addl) | sumh, suml

  {{ Add the 64-bit value `addh:`addl to the 64-bit value stored at `sumaddr.
  }}

  longmove(@sumh, sumaddr, 2)
  sumh += addh
  suml += addl
  if (_ult(suml, addl))
    sumh++
  longmove(sumaddr, @sumh, 2)

PRI _ult(x, y)

  {{ Test for unsigned `x < unsigned `y. Return `true if less than; `false, otherwise. 
  }}

  return x ^ $8000_0000 < y ^ $8000_0000